@@ -47,7 +47,6 @@
 #' @examples
 #' \donttest{
 #' library(CohortConstructor)
-#' if(isTRUE(omock::isMockDatasetDownloaded("GiBleed"))){
 #' cdm <- mockCohortConstructor()
 #'
 #' cdm$cohort <- conceptCohort(cdm = cdm, conceptSet = list(a = 444074), name = "cohort")
@@ -74,7 +73,6 @@
 #' cdm$study_cohort |>
 #'   attrition()
 #' }
-#' }
 conceptCohort <- function(cdm,
                           conceptSet,
                           name,
@@ -491,21 +489,12 @@ fulfillCohortReqs <- function(cdm, name, useRecordsBeforeObservation, type = "st
   return(cdm[[name]])
 }
 
-
-vocabVersion <- function(cdm) {
-  as.character(cdm$vocabulary |>
-                 dplyr::rename_with(tolower) |>
-                 dplyr::filter(.data$vocabulary_id == "None") |>
-                 dplyr::select("vocabulary_version") |>
-                 dplyr::collect())
-}
-
 conceptSetToCohortSet <- function(conceptSet, cdm) {
   cohSet <- dplyr::tibble("cohort_name" = names(conceptSet)) |>
     dplyr::mutate(
       "cohort_definition_id" = as.integer(dplyr::row_number()),
       "cdm_version" = attr(cdm, "cdm_version"),
-      "vocabulary_version" = vocabVersion(cdm)
+      "vocabulary_version" = CodelistGenerator::vocabularyVersion(cdm)
     )
   if (length(conceptSet) == 0) {
     cohSet <- cohSet |>
