@@ -1,43 +1,26 @@
 test_that("simple example", {
   skip_on_cran()
-  cdm <- omock::mockPerson(nPerson = 3)
-  cdm$observation_period <-  dplyr::tibble(
-    "observation_period_id" = c(1L, 2L, 3L),
-    "person_id" = c(1L, 2L, 3L),
-    "observation_period_start_date" = as.Date("2000-01-01"),
-    "observation_period_end_date" = as.Date("2024-01-01"),
-    "period_type_concept_id" = 1L
-  )
-  cdm <- omopgenerics::insertTable(cdm, name = "cohort",
-    table = dplyr::tibble(
+  cdm <- omock::mockCdmFromTables(tables = list(
+    observation_period = dplyr::tibble(
+      "observation_period_id" = c(1L, 2L, 3L),
+      "person_id" = c(1L, 2L, 3L),
+      "observation_period_start_date" = as.Date("2000-01-01"),
+      "observation_period_end_date" = as.Date("2024-01-01"),
+      "period_type_concept_id" = 1L
+    ),
+    cohort = dplyr::tibble(
       "cohort_definition_id" = 1L,
       "subject_id" = c(1L, 1L, 2L, 3L),
-      "cohort_start_date" = as.Date(c("2020-01-01",
-                                      "2020-01-12",
-                                      "2021-01-01",
-                                      "2022-01-01")),
-      "cohort_end_date" = as.Date(c("2020-01-10",
-                                    "2020-01-15",
-                                    "2021-01-01",
-                                    "2022-01-01"))
+      "cohort_start_date" = as.Date(c(
+        "2020-01-01", "2020-01-12", "2021-01-01", "2022-01-01"
+      )),
+      "cohort_end_date" = as.Date(c(
+        "2020-01-10", "2020-01-15", "2021-01-01", "2022-01-01"
+      ))
     )
-  )
-  cdm <- omopgenerics::insertTable(cdm, name = "cohort",
-    table = dplyr::tibble(
-      "cohort_definition_id" = 1L,
-      "subject_id" = c(1L, 1L, 2L, 3L),
-      "cohort_start_date" = as.Date(c("2020-01-01",
-                                      "2020-01-12",
-                                      "2021-01-01",
-                                      "2022-01-01")),
-      "cohort_end_date" = as.Date(c("2020-01-10",
-                                    "2020-01-15",
-                                    "2021-01-01",
-                                    "2022-01-01"))
-    )
-  )
-  cdm <- cdm |> copyCdm()
-  cdm$cohort <- omopgenerics::newCohortTable(cdm$cohort)
+  )) |>
+    copyCdm()
+
   expect_no_error(sameCohort <- cdm$cohort |>
                     collapseCohorts(gap = 0,
                                     name = "new_cohort"))
@@ -109,38 +92,38 @@ test_that("simple example", {
 
   expect_true(sum(grepl("og", omopgenerics::listSourceTables(cdm))) == 0)
 
-  PatientProfiles::mockDisconnect(cdm)
+  dropCreatedTables(cdm = cdm)
 })
 
 test_that("infitine", {
   skip_on_cran()
 
-  cdm <- omock::mockCdmFromTables()
-  cdm$person <- dplyr::tibble(
-    "person_id" = c(1L, 2L, 3L),
-    "gender_concept_id" = 1L,
-    "year_of_birth" = 1990L,
-    "race_concept_id" = 1L,
-    "ethnicity_concept_id" = 1L
-  )
-  cdm$observation_period <-  dplyr::tibble(
-    "observation_period_id" = c(1L, 2L, 3L),
-    "person_id" = c(1L, 2L, 3L),
-    "observation_period_start_date" = as.Date("2000-01-01"),
-    "observation_period_end_date" = as.Date("2024-01-01"),
-    "period_type_concept_id" = 1L
-  )
-  cdm$cohort <- dplyr::tibble(
-    "cohort_definition_id" = c(1L, 1L, 1L, 2L),
-    "subject_id" = c(1L, 2L, 3L, 3L),
-    "cohort_start_date" = as.Date(c("2020-01-01", "2020-01-01",
-                                  "2020-01-01", "2021-01-01")),
-    "cohort_end_date" = as.Date(c("2022-01-01", "2022-01-01",
-                                "2022-01-01", "2023-01-01"))
-  )
+  cdm <- omock::mockCdmFromTables(tables = list(
+    person = dplyr::tibble(
+      "person_id" = c(1L, 2L, 3L),
+      "gender_concept_id" = 1L,
+      "year_of_birth" = 1990L,
+      "race_concept_id" = 1L,
+      "ethnicity_concept_id" = 1L
+    ),
+    observation_period = dplyr::tibble(
+      "observation_period_id" = c(1L, 2L, 3L),
+      "person_id" = c(1L, 2L, 3L),
+      "observation_period_start_date" = as.Date("2000-01-01"),
+      "observation_period_end_date" = as.Date("2024-01-01"),
+      "period_type_concept_id" = 1L
+    ),
+    cohort = dplyr::tibble(
+      "cohort_definition_id" = c(1L, 1L, 1L, 2L),
+      "subject_id" = c(1L, 2L, 3L, 3L),
+      "cohort_start_date" = as.Date(c("2020-01-01", "2020-01-01",
+                                      "2020-01-01", "2021-01-01")),
+      "cohort_end_date" = as.Date(c("2022-01-01", "2022-01-01",
+                                    "2022-01-01", "2023-01-01"))
+    )
+  )) |>
+    copyCdm()
 
-  cdm <- cdm |> copyCdm()
-  cdm$cohort <- omopgenerics::newCohortTable(cdm$cohort)
   # for each person and each cohort id we should go from
   # first cohort start to last cohort entry
   cdm$cohort_collapsed <- cdm$cohort |>
@@ -180,53 +163,62 @@ test_that("infitine", {
       dplyr::pull("cohort_definition_id") == 2
   )
 
-
   expect_true(sum(grepl("og", omopgenerics::listSourceTables(cdm))) == 0)
-  PatientProfiles::mockDisconnect(cdm)
+
+  dropCreatedTables(cdm = cdm)
 })
 
 test_that("multiple observation periods", {
 # collapse should respect observation end dates
   skip_on_cran()
 
-  cdm <- omock::mockCdmReference() |>
-    omock::mockCdmFromTables(tables = list("cohort" = dplyr::tibble(
-      "cohort_definition_id" = 1L,
-      "subject_id" = 1L,
-      "cohort_start_date" = as.Date("2020-01-01"),
-      "cohort_end_date" = as.Date("2020-01-01")
-    )))
-  cdm <- omopgenerics::insertTable(
-    cdm = cdm, name = "concept", table = dplyr::tibble(
-      "concept_id" = 1L,
-      "concept_name" = "my concept",
-      "domain_id" = "drug",
-      "vocabulary_id" = NA_integer_,
-      "concept_class_id" = NA_integer_,
-      "concept_code" = NA_integer_,
-      "valid_start_date" = as.Date(NA),
-      "valid_end_date" = as.Date(NA)
+  cdm <- omopgenerics::cdmFromTables(
+    tables = list(
+      person = dplyr::tibble(
+        person_id = 1L,
+        gender_concept_id = 8507L,
+        year_of_birth = 1950L,
+        race_concept_id = 0L,
+        ethnicity_concept_id = 0L
+      ),
+      concept = dplyr::tibble(
+        "concept_id" = 1L,
+        "concept_name" = "my concept",
+        "domain_id" = "drug",
+        "vocabulary_id" = NA_character_,
+        "concept_class_id" = NA_character_,
+        "concept_code" = NA_character_,
+        "valid_start_date" = as.Date("1900-01-01"),
+        "valid_end_date" = as.Date("2100-01-01")
+      ),
+      vocabulary = omock:::mockVocabulary,
+      drug_exposure = dplyr::tibble(
+        "drug_exposure_id" = c(1L, 2L),
+        "person_id" = 1L,
+        "drug_concept_id" = 1L,
+        "drug_exposure_start_date" = as.Date(c("2020-01-01", "2021-01-01")),
+        "drug_exposure_end_date" =  as.Date(c("2020-01-15", "2021-01-15")),
+        "drug_type_concept_id" = 1L,
+        verbatim_end_date = drug_exposure_end_date
+      ),
+      observation_period = dplyr::tibble(
+        "observation_period_id" = c(1L, 2L),
+        "person_id" = 1L,
+        "period_type_concept_id" = 1L,
+        "observation_period_start_date" = as.Date(c("2020-01-01", "2021-01-01")),
+        "observation_period_end_date" =  as.Date(c("2020-06-01", "2021-06-01"))
+      )
+    ),
+    cdmName = "mock", cohortTables = list(
+      cohort = dplyr::tibble(
+        "cohort_definition_id" = 1L,
+        "subject_id" = 1L,
+        "cohort_start_date" = as.Date("2020-01-01"),
+        "cohort_end_date" = as.Date("2020-01-01")
+      )
     )
-  )
-  cdm <- omopgenerics::insertTable(
-    cdm = cdm, name = "drug_exposure", table = dplyr::tibble(
-      "drug_exposure_id" = c(1L, 2L),
-      "person_id" = 1L,
-      "drug_concept_id" = 1L,
-      "drug_exposure_start_date" = as.Date(c("2020-01-01", "2021-01-01")),
-      "drug_exposure_end_date" =  as.Date(c("2020-01-15", "2021-01-15")),
-      "drug_type_concept_id" = 1L
-    ))
-  cdm <- omopgenerics::insertTable(
-    cdm = cdm, name = "observation_period", table = dplyr::tibble(
-      "observation_period_id" = c(1L, 2L),
-      "person_id" = 1L,
-      "period_type_concept_id" = 1L,
-      "observation_period_start_date" = as.Date(c("2020-01-01", "2021-01-01")),
-      "observation_period_end_date" =  as.Date(c("2020-06-01", "2021-06-01"))
-    ))
-
-  cdm <- cdm |> copyCdm()
+  ) |>
+    copyCdm()
 
   expect_no_error(cdm$cohort_1 <- conceptCohort(cdm = cdm,
                                           conceptSet = list(a = 1L),
@@ -248,45 +240,79 @@ test_that("multiple observation periods", {
 
   expect_true(sum(grepl("og", omopgenerics::listSourceTables(cdm))) == 0)
 
-  PatientProfiles::mockDisconnect(cdm)
+  if (dbToTest == "postgres CDMConnector") {
+    con <- CDMConnector::cdmCon(cdm = cdm)
 
-})
-
-test_that("test indexes - postgres", {
-  skip_on_cran()
-  skip_if(Sys.getenv("CDM5_POSTGRESQL_DBNAME") == "")
-  skip_if(!testIndexes)
-
-  db <- DBI::dbConnect(RPostgres::Postgres(),
-                       dbname = Sys.getenv("CDM5_POSTGRESQL_DBNAME"),
-                       host = Sys.getenv("CDM5_POSTGRESQL_HOST"),
-                       user = Sys.getenv("CDM5_POSTGRESQL_USER"),
-                       password = Sys.getenv("CDM5_POSTGRESQL_PASSWORD"))
-  cdm <- CDMConnector::cdmFromCon(
-    con = db,
-    cdmSchema = Sys.getenv("CDM5_POSTGRESQL_CDM_SCHEMA"),
-    writeSchema = Sys.getenv("CDM5_POSTGRESQL_SCRATCH_SCHEMA"),
-    writePrefix = "cc_",
-    achillesSchema = Sys.getenv("CDM5_POSTGRESQL_CDM_SCHEMA")
-  )
-
-  omopgenerics::dropSourceTable(cdm = cdm, name = dplyr::contains("og_"))
-
-  cdm <- omopgenerics::insertTable(cdm = cdm,
-                                   name = "my_cohort",
-                                   table = data.frame(cohort_definition_id = 1L,
-                                                      subject_id = 1L,
-                                                      cohort_start_date = as.Date("2009-01-01"),
-                                                      cohort_end_date = as.Date("2009-01-02")))
-  cdm$my_cohort <- omopgenerics::newCohortTable(cdm$my_cohort)
-  cdm$my_cohort <- collapseCohorts(cdm$my_cohort)
-
-  expect_true(
-    DBI::dbGetQuery(db, paste0("SELECT * FROM pg_indexes WHERE tablename = 'cc_my_cohort';")) |> dplyr::pull("indexdef") ==
+    # check indexes are still there
+    indexdef <- dplyr::tbl(con, "pg_indexes") |>
+      dplyr::filter(.data$table_name == "coco_test_cohort_1") |>
+      dplyr::pull("indexdef")
+    expect_indentical(
+      indexdef,
       "CREATE INDEX cc_my_cohort_subject_id_cohort_start_date_idx ON public.cc_my_cohort USING btree (subject_id, cohort_start_date)"
-  )
+    )
+  }
 
-  expect_true(sum(grepl("og", omopgenerics::listSourceTables(cdm))) == 0)
-  omopgenerics::dropSourceTable(cdm = cdm, name = dplyr::starts_with("my_cohort"))
-  CDMConnector::cdmDisconnect(cdm = cdm)
+  dropCreatedTables(cdm = cdm)
+})
+
+test_that("overlapping input cohort", {
+  skip_on_cran()
+  cdm <- omock::mockCdmFromTables(tables = list(
+    observation_period = dplyr::tibble(
+      "observation_period_id" = c(1L, 2L, 3L),
+      "person_id" = c(1L, 2L, 3L),
+      "observation_period_start_date" = as.Date("2000-01-01"),
+      "observation_period_end_date" = as.Date("2024-01-01"),
+      "period_type_concept_id" = 1L
+    ))) |>
+    copyCdm()
+
+  # overlap should still work if input cohort has overlaps
+  cdm <- omopgenerics::insertTable(
+    cdm = cdm,
+    name = "cohort",
+    table = dplyr::tibble(
+      "cohort_definition_id" = 1L,
+      "subject_id" = c(1L, 1L, 2L, 3L),
+      "cohort_start_date" = as.Date(c(
+        "2020-01-01", "2020-01-08", "2021-01-01", "2022-01-01"
+      )),
+      "cohort_end_date" = as.Date(c(
+        "2020-01-10", "2020-01-15", "2021-01-01", "2022-01-01"
+      ))
+    ))
+  cdm$cohort <- omopgenerics::newCohortTable(cdm$cohort, .softValidation = TRUE)
+
+  expect_no_error(cdm$collapsed_cohort <- cdm$cohort |>
+                    collapseCohorts(gap = 0,
+                                    name = "collapsed_cohort"))
+  expect_true(cdm$collapsed_cohort |>
+    dplyr::filter(subject_id == 1) |>
+    dplyr::pull(cohort_start_date) == "2020-01-01")
+  expect_true(cdm$collapsed_cohort |>
+    dplyr::filter(subject_id == 1) |>
+    dplyr::pull(cohort_end_date) == "2020-01-15")
+
+  expect_no_error(cdm$collapsed_cohort <- cdm$cohort |>
+                    collapseCohorts(gap = 1,
+                                    name = "collapsed_cohort"))
+  expect_true(cdm$collapsed_cohort |>
+                dplyr::filter(subject_id == 1) |>
+                dplyr::pull(cohort_start_date) == "2020-01-01")
+  expect_true(cdm$collapsed_cohort |>
+                dplyr::filter(subject_id == 1) |>
+                dplyr::pull(cohort_end_date) == "2020-01-15")
+
+  expect_no_error(cdm$collapsed_cohort <- cdm$cohort |>
+                    collapseCohorts(gap = Inf,
+                                    name = "collapsed_cohort"))
+  expect_true(cdm$collapsed_cohort |>
+                dplyr::filter(subject_id == 1) |>
+                dplyr::pull(cohort_start_date) == "2020-01-01")
+  expect_true(cdm$collapsed_cohort |>
+                dplyr::filter(subject_id == 1) |>
+                dplyr::pull(cohort_end_date) == "2020-01-15")
+
+ dropCreatedTables(cdm = cdm)
 })
