@@ -128,7 +128,7 @@ test_that("unionCohorts works", {
 
   # test gap and keep original cohorts ----
   cdm$cohort7 <- unionCohorts(
-    cdm$cohort1, name = "cohort7", gap = 10000, keepOriginalCohorts = TRUE, cohortId = c(1:3, 5)
+    cdm$cohort1, name = "cohort7", gap = Inf, keepOriginalCohorts = TRUE, cohortId = c(1:3, 5)
   )
   expect_equal(collectCohort(cdm$cohort1, 1), collectCohort(cdm$cohort7, 1))
   expect_equal(collectCohort(cdm$cohort1, 2), collectCohort(cdm$cohort7, 2))
@@ -142,6 +142,14 @@ test_that("unionCohorts works", {
       cohort_end_date = as.Date(c("2011-01-01", "2002-06-07", "2015-06-01"))
     )
   )
+  expect_equal(
+    settings(cdm$cohort7) |> dplyr::filter(cohort_definition_id == 6),
+    dplyr::tibble(
+      cohort_definition_id = 6L,
+      cohort_name = "cohort_1_cohort_2_cohort_3_cohort_5",
+      gap = Inf
+    )
+  )
 
   # Indexes ----
   if (dbToTest == "postgres CDMConnector" & testIndexes) {
@@ -178,13 +186,6 @@ test_that("unionCohorts works", {
                            cohortName = NULL,
                            name = "cohort1")
   )
-  expect_error(
-    cohort <- unionCohorts(cdm$cohort1,
-                           cohortId = NULL,
-                           gap = Inf,
-                           cohortName = NULL,
-                           name = "cohort1")
-  )
   expect_error(
     cohort <- unionCohorts(cdm$cohort1,
                            cohortId = "1",
